""" Add FX to selected events """

# ***************************************************** 
# * Author: Harold Linke
# * Date: October 12, 2018
# *
# * History of Change
# * ***************************************************
# * * Version * dd/mm/yyyy * Comment
# * ***************************************************
# * * 1.0     * 07/10/2018 * inital creation based on JScript script 
# * *         *            * by: Joe Howes (joeh@guild1.com)
# * *         *            * Last Modification: 2004
# * ***************************************************
# * *         *            * 
# * *         *            * 
# * ***************************************************
# * *         *            * 
# * *         *            * 
# * *         *            * 
#******************************************************

import clr
clr.AddReference('System.Windows.Forms')
import System.Windows.Forms
from System.Windows.Forms import *

clr.AddReference('System.Drawing')
import System.Drawing

clr.AddReference('ScriptPortal.Vegas')
import ScriptPortal.Vegas
from ScriptPortal.Vegas import *

#clr.AddReference("System.Text")
#import System.Text;
#clr.AddReference("System.Collections")
#import System.Collections;


#/*
# * Copyright (c) 2004, Folding Rain Filmworks Inc.
# *
# * author: Joe Howes (joeh@guild1.com)
# *
# * Coloring your final project in Vegas can be a bit of a bitch,
# * espcially if you have a complex project divided into multiple
# * sections.  When you're going through and coloring, you may find
# * that as you progress, coloring decisions you made early on
# * just look lame, and going back and updating all of them is a
# * pain in the ass because Vegas doesn't allow you to apply
# * changes to multiple events or media at the same time...you have
# * to go into each one individually.
# *
# * self script allows you to select one or more video events on the
# * timeline and apply effects and presets to either the events
# * themselves, or the media in the media pool.  self script has
# * saved me VAST amounts of time in finishing, and I hope it does
# * the same for you.
# *
# * NOTE:
# * - The reason you can't select items in the media pool and affect
# *   them directly is that in self version of Vegas (5.0b), the
# *   the scripting API doesn't allow you to check whether an item in
# *   the media pool is selected, so it would have to apply the effects
# *   to ALL media in the pool.
# *
# * - When affecting media, the effects are applied to all takes.
# *
# * - When affecting media, the algorithm is smart enough to check
# *   if a media pool item has already been affected and only process
# *   each one once.
# *
# * USAGE:
# * Select one or more video events on the timeline and run the script.
# * From the dialog choose one effect and (optional) preset at a time
# * and click "Add Effect" to add it to the stack of effects that will
# * be applied to all selected events or media.  Choose whether to
# * affect events or media, then click "Assign".
# *
# * v1.1: Sep. 24, 2004
# *
# * VERSION HISTORY:
# *
# * 1.1:
# * - Added the option to add to existing FX as well as replace.
# * - (Hopefully) fixed some GUI display issues...where labels were
# *   being truncated.
# */

#/**
# * CLASS: FXReplacerDialog
# * The GUI.
# */
class FXReplacerDialog(Form):
#
    videoFXListBox = []            # All Vegas' video FX will be listed here
    fxPresetListBox = []            # All the presets
    finalApplicationListBox = []    # All the user's choices
    plugInList = []
#
#    applyToEventsRadio : RadioButton;
#    applyToMediaRadio : RadioButton;
#    replaceExistingFXCheck : CheckBox;        # Replace or add to FX chain?
#
#    addFXButton : Button;
#    removeFXButton : Button;
#
#    assignButton : Button;
#    cancelButton : Button;
#
#    videoFXParamGroup : GroupBox;
#    fxPresetParamGroup : GroupBox;
#    finalApplicationsParamGroup : GroupBox;
#
    presetList = []               # An ArrayList of ArrayLists holding presets
#                                            # for each plug-in
    plugInNameArray = []        # Contains the user's chosen plug-in names
    plugInPresetArray = []        # Contains the user's chosen preset names
    removeEnabled = True

#    /**
#     * FUNCTION: populate
#     * Populates the data model with all of Vegas' plug-ins and presets.
#     */
    def populate(self):

        fx = pyVEGAS.VideoFX
 

        first = True;
        self.videoFXListBox.BeginUpdate();

        # Get all the plug-ins
        for plugIn in fx: 
            if (first):
                first = False;
                continue;
            
            self.plugInList.Add(plugIn.Name);

        self.plugInList.sort();

        # Populate the videoFX list box and the presets data model
        
        for plugInName in self.plugInList:
            self.videoFXListBox.Items.Add(plugInName);
            plugIn = fx.GetChildByName(plugInName);
            presets = []

            for preset in plugIn.Presets:
                presets.Add(preset.Name)
 
            self.presetList.Add(presets)

        self.videoFXListBox.EndUpdate();

#    /**
#     * FUNCTION: videoFXListBoxOnClick
#     * Event handler for clicks on the videoFXListBox.  We update GUI buttons,
#     * populate the preset box.
#     */
    def videoFXListBoxOnClick(self,sender, evt) :

        index = self.videoFXListBox.SelectedIndex;

        # If there's at least one item selected in the list, enable the
        # Add Effect button.
        if (index != -1):
            self.addFXButton.Enabled = True;
        


        # Get the presets for the chosen plug-in and populate the presets box.
        presets = self.presetList[index]

        self.fxPresetListBox.BeginUpdate();
        self.fxPresetListBox.Items.Clear();

        for preset in presets:
           self.fxPresetListBox.Items.Add(preset);
        
        self.fxPresetListBox.EndUpdate();

#    /**
#     * FUNCTION: addFXOnClick
#     * When the Add Effect button is clicked, reflect the user's choice
#     * both in the GUI and in the underlying data model.
#     */
    def addFXOnClick(self, sender, evt):

        # Add the plug-in choice
        plugInName = self.videoFXListBox.SelectedItem
        presetIndex = self.fxPresetListBox.SelectedIndex
        plugInPreset = None
        if (presetIndex != -1):
            plugInPreset = self.fxPresetListBox.SelectedItem


        displayString = plugInName

        # If a preset is chosen, add that too
        if (presetIndex != -1):
            displayString += " - <"
            displayString += plugInPreset
            displayString += ">"

        self.finalApplicationListBox.Items.Add(displayString)

        # Add the plug-in choice to the data model
        self.plugInNameArray.Add(plugInName)

        # If there is a preset choice, add that, otherwise add null
        if (presetIndex != -1):
            self.plugInPresetArray.Add(plugInPreset)
        else:
            self.plugInPresetArray.Add(None)
        

        # If there is at least one item available in the FX to Assign box,
        # enable the Remove Effect button
        self.removeEnabled = (self.finalApplicationListBox.Items.Count > 0)
        self.removeFXButton.Enabled = self.removeEnabled;

#    /**
#     * FUNCTION: removeFXOnClick
#     * Remove the selected cohice from the FX to Assign box and from the data model.
#     */
    def removeFXOnClick(self, sender, evt ):

        # Remove from the data model
        self.plugInNameArray.RemoveAt(self.finalApplicationListBox.SelectedIndex);
        self.plugInPresetArray.RemoveAt(self.finalApplicationListBox.SelectedIndex);

        # Remove from the GUI
        self.finalApplicationListBox.BeginUpdate();
        self.finalApplicationListBox.Items.Remove(self.finalApplicationListBox.SelectedItem);
        self.finalApplicationListBox.EndUpdate();
        self.removeEnabled = (self.finalApplicationListBox.Items.Count > 0)
        self.removeFXButton.Enabled = self.removeEnabled;

#    /**
#     * FUNCTION: getPlugInNameArray
#     * Accessor for the user's plug-in choices.
#     */
    def getPlugInNameArray(self):
        return self.plugInNameArray

#    /**
#     * FUNCTION: getPlugInPresetArray
#     * Accessor for the user's preset choices.
#     */
    def getPlugInPresetArray(self):
        return self.plugInPresetArray

#    /**
#     * CONSTRUCTOR
#     * Init everything.
#     */
    def __init__(self):

        self.Text = "Assign Plug-Ins to Selected Events or their Media"
        self.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
        self.MaximizeBox = False;
        self.StartPosition = FormStartPosition.CenterScreen;

#        plugInNameArray = ArrayList();
#        plugInPresetArray = ArrayList();

        self.Width = 470;
        self.Height = 520;


        # VideoFX
        self.videoFXListBox = ListBox()
        self.videoFXListBox.Location = System.Drawing.Point(8, 18)
        self.videoFXListBox.Name = "videoFXListBox";
        self.videoFXListBox.Size = System.Drawing.Size(200, 200);
        self.videoFXListBox.TabIndex = 1;
        self.videoFXListBox.add_Click(self.videoFXListBoxOnClick)
        
        # VideoFX ParamGroup
        self.videoFXParamGroup = GroupBox();
        self.videoFXParamGroup.Controls.Add(self.videoFXListBox);
        self.videoFXParamGroup.Location = System.Drawing.Point(8, 8);
        self.videoFXParamGroup.Name = "videoFXParamGroup";
        self.videoFXParamGroup.Size = System.Drawing.Size(216, 225);
        #videoFXParamGroup.TabIndex = -1;
        self.videoFXParamGroup.TabStop = False;
        self.videoFXParamGroup.Text = "Choose a Plug-In";

        self.Controls.Add(self.videoFXParamGroup);


        # FX Presets
        self.fxPresetListBox = ListBox();
        self.fxPresetListBox.Location = System.Drawing.Point(8, 18);
        self.fxPresetListBox.Name = "fxPresetListBox";
        self.fxPresetListBox.Size = System.Drawing.Size(200, 200);
        self.fxPresetListBox.TabIndex = 2;

        # FX Presets ParamGroup
        self.fxPresetParamGroup = GroupBox();
        self.fxPresetParamGroup.Controls.Add(self.fxPresetListBox);
        self.fxPresetParamGroup.Location = System.Drawing.Point(240, 8);
        self.fxPresetParamGroup.Name = "fxPresetParamGroup";
        self.fxPresetParamGroup.Size = System.Drawing.Size(216, 225);
        #fxPresetParamGroup.TabIndex = -1;
        self.fxPresetParamGroup.TabStop = False;
        self.fxPresetParamGroup.Text = "Choose a Preset (optional)";

        self.Controls.Add(self.fxPresetParamGroup);


        # Final Applications
        self.finalApplicationListBox = ListBox();
        self.finalApplicationListBox.Location = System.Drawing.Point(8, 18);
        self.finalApplicationListBox.Name = "finalApplicationsListBox";
        self.finalApplicationListBox.Size = System.Drawing.Size(400, 100);
        self.finalApplicationListBox.TabIndex = 5;

        # Replace or Add?
        self.replaceExistingFXCheck = CheckBox();
        self.replaceExistingFXCheck.Location = System.Drawing.Point(25, 125);
        self.replaceExistingFXCheck.Size = System.Drawing.Size(125, 18);
        self.replaceExistingFXCheck.Name = "replaceExistingFXCheck";
        self.replaceExistingFXCheck.TabIndex = 6;
        self.replaceExistingFXCheck.Checked = True;
        self.replaceExistingFXCheck.Text = "Replace Existing FX";

        # Events or Media Radios
        self.applyToEventsRadio = RadioButton();
        self.applyToEventsRadio.Location = System.Drawing.Point(190, 125);
        self.applyToEventsRadio.Size = System.Drawing.Size(110, 18);
        self.applyToEventsRadio.Name = "applytoEventsRadio";
        self.applyToEventsRadio.TabIndex = 7;
        self.applyToEventsRadio.Checked = True;
        self.applyToEventsRadio.Text = "Apply to Events";

        self.applyToMediaRadio = RadioButton();
        self.applyToMediaRadio.Location = System.Drawing.Point(300, 125);
        self.applyToMediaRadio.Size = System.Drawing.Size(110, 18);
        self.applyToMediaRadio.Name = "applyToMediaRadio";
        self.applyToMediaRadio.TabIndex = 8;
        self.applyToMediaRadio.Checked = False;
        self.applyToMediaRadio.Text = "Apply to Media";

        # Final Applications ParamGroup
        self.finalApplicationsParamGroup = GroupBox();
        self.finalApplicationsParamGroup.Controls.Add(self.finalApplicationListBox);
        self.finalApplicationsParamGroup.Controls.Add(self.replaceExistingFXCheck);
        self.finalApplicationsParamGroup.Controls.Add(self.applyToEventsRadio);
        self.finalApplicationsParamGroup.Controls.Add(self.applyToMediaRadio);
        self.finalApplicationsParamGroup.Location = System.Drawing.Point(25, 270);
        self.finalApplicationsParamGroup.Name = "finalApplicationsParamGroup";
        self.finalApplicationsParamGroup.Size = System.Drawing.Size(416, 150);
        #finalApplicationsParamGroup.TabIndex = 4;
        self.finalApplicationsParamGroup.TabStop = False;
        self.finalApplicationsParamGroup.Text = "FX to Assign";

        self.Controls.Add(self.finalApplicationsParamGroup);


        # Add Effect button
        self.addFXButton = Button();
        self.addFXButton.Text = "Add Effect";
        self.addFXButton.Left = 124;
        self.addFXButton.Top = 240;
        self.addFXButton.Width = 100;
        self.addFXButton.add_Click(self.addFXOnClick);
        self.addFXButton.Enabled = True;
        self.addFXButton.TabIndex = 3;
        self.Controls.Add(self.addFXButton);

        # Remove Effect button
        self.removeFXButton = Button();
        self.removeFXButton.Text = "Remove Effect";
        self.removeFXButton.Left = 240;
        self.removeFXButton.Top = 240;
        self.removeFXButton.Width = 100;
        self.removeFXButton.add_Click(self.removeFXOnClick);
        self.removeFXButton.Enabled = True;
        self.removeFXButton.TabIndex = 4;
        self.Controls.Add(self.removeFXButton);

        # Assign button
        self.assignButton = Button();
        self.assignButton.DialogResult = System.Windows.Forms.DialogResult.OK;
        self.assignButton.Text = "Assign";
        self.assignButton.Left = 300;
        self.assignButton.Top = 425;
        self.AcceptButton = self.assignButton;
        self.assignButton.TabIndex = 9
        self.Controls.Add(self.assignButton);

        # Cancel button
        self.cancelButton = Button();
        self.cancelButton.DialogResult = System.Windows.Forms.DialogResult.Cancel;
        self.cancelButton.Text = "Cancel";
        self.cancelButton.Left = 380;
        self.cancelButton.Top = 425;
        self.CancelButton = self.cancelButton;
        self.cancelButton.TabIndex = 10;
        self.Controls.Add(self.cancelButton);


        self.populate();

#/**
# * FUNCTION: replaceMediaFX
# * Assign all the plug-ins from plugInArray, along with the presets in presetArray,
# * to the media entity.  Note that if the user chose not to specify a preset for a
# * given plug-in, it's corresponding presetArray entry will be a null.
# */
def replaceMediaFX(media, plugInArray, presetArray, replaceFX):

    if replaceFX:
        count = media.Effects.Count

        # Kill all the existing FX
        while (count > 0):
            media.Effects.RemoveAt(0)
            count = media.Effects.Count

    # Assign all the new FX
    #e2 = Enumerator(presetArray)
    count = 0
    for plugin in plugInArray:
        effect = Effect(plugIn)
        media.Effects.Add(effect)

        preset = presetArray[count]
        count += 1
 
        if (null != preset):
            effect.Preset = preset



#/**
# * FUNCTION: replaceEventFX
# * Assign all the plug-ins from plugInArray, along with the presets in presetArray,
# * to the event entity.  Note that if the user chose not to specify a preset for a
# * given plug-in, it's corresponding presetArray entry will be a null.
# */
def replaceEventFX(evnt, plugInArray, presetArray, replaceFX):

    if (replaceFX):
        count = evnt.Effects.Count

        # Kill all the existing FX
        while (count > 0):
            evnt.Effects.RemoveAt(0);
            count = evnt.Effects.Count


    # Assign all the new FX
    # var e = new Enumerator(plugInArray);
    # var e2 = new Enumerator(presetArray);
    count = 0
    for plugIn in plugInArray:
        effect = Effect(plugIn);
        evnt.Effects.Add(effect);

        preset = presetArray[count]
        count +=1

        if (None != preset):
            effect.Preset = preset


#
# * FUNCTION: replaceSelectedEventFX
# * Iterate through all the video events and, if selected, remove all their
# * FX plugins and replace with the user's selections.
#
def replaceSelectedEventFX(plugInFXArray, plugInPresetArray, replaceFX):

    done = False
    for track in pyVEGAS.Project.Tracks:
        if not track.IsVideo():
            continue

        for evnt in track.Events:
            if evnt.Selected:
                replaceEventFX(evnt, plugInFXArray, plugInPresetArray, replaceFX)
                done = True
    
    return done

#/**
# * FUNCTION: replaceSelectedMediaFX
# * Iterate through all the video events and, if selected, remove all the plug-ins
# * associated with their media pool entries and replace with the user's selections.
# */
def replaceSelectedMediaFX(plugInArray, presetArray, replaceFX):

    done = False;
    # affectedMedia = ArrayList();
    for track in Vegas.Project.Tracks:

        if not track.IsVideo():
            continue;

        for evnt in track.Events:

            if (evnt.Selected):
               for take in evnt.Takes:

                # Iterate through ALL the takes for self event and replace
                # the FX on their media pool entries.
                   media = take.Media;
                   if affectedMedia.Contains(media):
                       continue

                   replaceMediaFX(media, plugInFXArray, plugInPresetArray, replaceFX)
                   affectedMedia.Add(media)
                   done = True
    return done





fx = pyVEGAS.VideoFX;                # Vegas effects list
done = False;                    # If nothing is done, let the user know
#plugInNameArray : ArrayList;    # PlugIn names the user has chosen
#plugInFXArray : ArrayList;        # The plugins associated with those names
#plugInPresetArray : ArrayList;    # The presets associated with those plugins
dlog = FXReplacerDialog();    # The GUI


# Init
#plugInFXArray = ArrayList();


# Show the GUI
if DialogResult.OK == dlog.ShowDialog(): 

    # Get the user's choices
    plugInNameArray = dlog.getPlugInNameArray()
    plugInPresetArray = dlog.getPlugInPresetArray()
    plugInFXArray =[]

    # Load the plugins the user has chosen
    for plugInName in plugInNameArray:
        plugIn = fx.GetChildByName(plugInName)
        if plugIn==None: 
            print("could not find a plug-in named: '" + plugInName + "'")
        else:    
            plugInFXArray.Add(plugIn)

        # Pass those plugins and the user's preset choices to either
        # the event replacer or the media replacer
    replaceFX = dlog.replaceExistingFXCheck.Checked;
    if (dlog.applyToEventsRadio.Checked):
        done = replaceSelectedEventFX(plugInFXArray, plugInPresetArray, replaceFX)
    else:
        done = replaceSelectedMediaFX(plugInFXArray, plugInPresetArray, replaceFX)




